package pw.yumc.DataStorage;

import java.sql.ResultSet;
import java.sql.SQLException;

import org.bukkit.Bukkit;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import org.bukkit.plugin.java.JavaPlugin;

import me.clip.placeholderapi.PlaceholderAPI;
import me.clip.placeholderapi.PlaceholderHook;
import pw.yumc.YumCore.commands.CommandSub;
import pw.yumc.YumCore.commands.annotation.Cmd;
import pw.yumc.YumCore.commands.annotation.Help;
import pw.yumc.YumCore.commands.annotation.Sort;
import pw.yumc.YumCore.commands.interfaces.Executor;
import pw.yumc.YumCore.config.FileConfig;
import pw.yumc.YumCore.sql.DataBase;
import pw.yumc.YumCore.sql.core.KeyValue;

public class DataStorage extends JavaPlugin implements Executor {
    FileConfig config;
    DataBase db;
    String kfields = "key";
    String tableName;
    String vfields = "value";

    @Override
    public void onLoad() {
        config = new FileConfig();
    }

    @Override
    public void onEnable() {
        initDataBase();
        hookPAPI();
        new CommandSub("dd", this);
    }

    private void initDataBase() {
        db = DataBase.create(this, config.getConfigurationSection("Data"));
        tableName = config.getMessage("Data.MySQL.tablename");
        if (db.testConnect()) {
            this.getLogger().info("数据库连接成功,检查数据表是否存在...");
            if (!db.isTableExists(tableName)) {
                this.getLogger().info("数据表不存在,新建表" + tableName + "...");
                if (!db.createTables(tableName, new KeyValue(kfields, "VARCHAR(30) PRIMARY KEY").add(vfields, "INT"), "UNIQUE (`key`)")) {
                    this.getLogger().info("数据表" + tableName + "创建失败,请尝试手动创建并重启服务器...");
                }
            }
        } else {
            this.getLogger().warning("数据库连接失败!");
            this.getServer().getPluginManager().disablePlugin(this);
        }
    }

    private void hookPAPI() {
        final boolean result = PlaceholderAPI.registerPlaceholderHook("dd", new PlaceholderHook() {
            @Override
            public String onPlaceholderRequest(final Player arg0, final String arg1) {
                return db.dbSelectFirst(tableName, vfields, new KeyValue(kfields, arg1));
            }
        });
        if (result) {
            this.getLogger().info("发现 PlaceholderAPI 已关联...");
        } else {
            Bukkit.getConsoleSender().sendMessage("§6[§bDataStorage§6] §c发现 PlaceholderAPI 但是关联失败了!!!失败了!!!");
        }
    }

    @Sort(0)
    @Cmd
    @Help(value = "创建一个键值对", possibleArguments = "<键> <值>")
    public void create(CommandSender sender, String key, int value) {
        try {
            if (createValue(key, value)) {
                sender.sendMessage("§a成功创建了 §3" + key + " §a初始值为 §d" + value);
                return;
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }
        sender.sendMessage("§c操作失败 请查看后台错误 (添加删除需要初始化)!");
    }

    @Sort(3)
    @Cmd
    @Help(value = "移除一个键值对", possibleArguments = "<键>")
    public void remove(CommandSender sender, String key, int value) {
        try {
            if (removeValue(key)) {
                sender.sendMessage("§a成功移除了 §3" + key);
                return;
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }
        sender.sendMessage("§c操作失败 请查看后台错误 (添加删除需要初始化)!");
    }

    @Sort(1)
    @Cmd
    @Help(value = "键key的值添加value", possibleArguments = "<键> <值>")
    public void add(CommandSender sender, String key, int value) {
        try {
            if (incValue(key, value)) {
                sender.sendMessage("§a成功给了 §3" + key + " §a增加了 §e" + value + " §a目前的值为 §d" + getValue(key));
                return;
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }
        sender.sendMessage("§c操作失败 请查看后台错误 (添加删除需要初始化)!");
    }

    @Sort(2)
    @Cmd
    @Help(value = "键key的值添加value", possibleArguments = "<键> <值>")
    public void del(CommandSender sender, String key, int value) {
        try {
            if (decValue(key, value)) {
                sender.sendMessage("§a成功给了 §3" + key + " §c减少了 §e" + value + " §a目前的值为 §d" + getValue(key));
                return;
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }
        sender.sendMessage("§c操作失败 请查看后台错误 (添加删除需要初始化)!");
    }

    private boolean createValue(String key, int value) throws SQLException {
        return db.getDataBaseCore().update("INSERT INTO " + tableName + " (`" + kfields + "`, `" + vfields + "`) VALUES ('" + key + "', '" + value + "')") != 0;
    }

    private boolean removeValue(String key) throws SQLException {
        return db.getDataBaseCore().update("DELETE FROM " + tableName + " WHERE `" + kfields + "` = '" + key + "'") != 0;
    }

    private boolean incValue(String key, int value) throws SQLException {
        return db.getDataBaseCore().update("UPDATE " + tableName + " SET `" + vfields + "` = " + vfields + " + " + value + " WHERE `" + kfields + "` = '" + key + "'") != 0;
    }

    private boolean decValue(String key, int value) throws SQLException {
        return db.getDataBaseCore().update("UPDATE " + tableName + " SET `" + vfields + "` = `" + vfields + "` - " + value + " WHERE `" + kfields + "` = '" + key + "'") != 0;
    }

    private int getValue(String key) throws SQLException {
        ResultSet result = db.getDataBaseCore().query("SELECT `" + vfields + "` FROM " + tableName + " WHERE `" + kfields + "` = '" + key + "' LIMIT 1");
        if (result.next()) { return result.getInt(1); }
        return 0;
    }

    @Override
    public void onDisable() {
        if (db != null) {
            db.close();
        }
        PlaceholderAPI.unregisterPlaceholderHook("dd");
    }

}
